"use strict";

let menu;
//let pos = "tl";

createMenu();

async function createMenu() {
  if (!menu) {
    let doc = (document.body || document.documentElement);
    if (doc) {
      // 一時的に <aside> 要素を参照
      const asideMenu = document.createElement("aside");
      asideMenu.style.cssText = "background: transparent; padding: 0; margin: 0; border: 0; opacity: 1; position: absolute; top: -100%";
      doc.appendChild(asideMenu);

      let shadow = asideMenu.attachShadow({ mode: "closed" });
      let link = document.createElement("link");
      link.rel = "stylesheet";
      link.href = chrome.runtime.getURL("inject.css");
      shadow.appendChild(link);

      // <aside> 要素を menu 変数に格納
      menu = shadow.appendChild(document.createElement("nav"));
      await addCSS();

      // 一時的な <aside> 要素を削除
      asideMenu.remove();
    } else {
      return;
    }

    menu.className = "corner_tl";
    menu.textContent = "";

    // 保存されているボタン設定を取得
    chrome.storage.sync.get({ buttonSettings: getDefaultButtonTexts(), buttonCount: 15 }, data => {
      // 修正点: コールバック関数内で data オブジェクトを確認
      const buttonCount = data.buttonCount;
      console.log("bcount=", buttonCount);

      const bottomLinkButtons = createButtonContainer("bottomLinkButtons", "bottomLinkButtons", document);
      for (let i = 1; i <= 5; i++) {
        const linkText = i != null ? data.buttonSettings[i - 1]?.text : null;
        const linkAction = `link${i}`;
        createButton(linkText, linkAction, bottomLinkButtons);
      }

      const leftLinkButtonsContainer = createButtonContainer("leftLinkButtons", "leftLinkButtons", document);
      for (let i = 6; i <= buttonCount; i++) {
        const linkText = i != null ? data.buttonSettings[i - 1]?.text : null;
        const linkAction = `link${i}`;
        createButton(linkText, linkAction, leftLinkButtonsContainer);
      }
    });


    // <aside> 要素にクラスを追加
    menu.classList.add("asideMenu");

    function addCSS() {
      fetch(chrome.runtime.getURL("inject.css"))
        .then(response => response.text())
        .then(cssText => {
          const style = document.createElement("style");
          style.textContent = cssText;
          document.head.appendChild(style);
        });
    }


    function createButtonContainer(id, className, parent) {
      let container = parent === document ? parent.getElementById(id) : null;
      // 修正点: parent が document の場合のみ getElementById を呼ぶ
      if (!container) {
        const body = document.body || document.documentElement;
        if (!body) {
          console.error("Document body is null or undefined.");
          return null;
        }
        container = document.createElement("div");
        container.id = id;
        container.classList.add(className);
        body.appendChild(container);
      }
      return container;
    }


    function createButton(text, action, container) {
      const button = createElement("div", "customButton", { "data-action": action }, text);
      container.appendChild(button);
      button.addEventListener('click', handleSelection);
      return button;
    }
  }
}


function createElement(tag, klass, attr, text) {
  let n = document.createElement(tag);
  n.setAttribute("role", "button");
  if (klass) n.className = klass;
  if (attr) for (let prop in attr) n.setAttribute(prop, attr[prop]);
  if (text) n.textContent = text;
  return n;
}

// ボタン押下時処理
async function handleSelection(e, selection) {
  let n = selection || e.target;
  let label = n && n.getAttribute && n.getAttribute("data-action");

  // オプション設定取得
  const waitTime = await getWaitTime();
  const recoveryOption = await getRecoveryOption();
  const data = await new Promise(resolve => {
    chrome.storage.sync.get({ buttonSettings: getDefaultButtonTexts() }, data => {
      resolve(data);
    });
  });

  if (label) {
    const buttonNumber = extractNumber(label);
    const jumpUrl = buttonNumber != null ? data.buttonSettings[buttonNumber - 1]?.url : null;

    if (jumpUrl && recoveryOption === "on") {
      console.log("回復実行 wait=", waitTime);
      window.location.assign("http://sp.pf.mbga.jp/12010355/?guid=ON&url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fbattle%2Fability_execute.ashx%3Faid%3D30%3D10&guid=ON");
      await bsleep(waitTime);
    }

    if (jumpUrl) {
      console.log("jumpUrl=", jumpUrl);
      window.location.assign(jumpUrl);
    }
  }

  function extractNumber(label) {
    const match = label.match(/\d+/);
    return match ? parseInt(match[0]) : null;
  }
}


function getWaitTime() {
  return new Promise(resolve => {
    chrome.storage.sync.get({ waitTime: 300 }, obj => {
      resolve(obj.waitTime);
    });
  });
}

function getRecoveryOption() {
  return new Promise(resolve => {
    chrome.storage.sync.get({ recoveryOption: "on" }, obj => {
      resolve(obj.recoveryOption);
    });
  });
}

async function bsleep(waitMsec) {
  var startMsec = new Date();
  while (new Date() - startMsec < waitMsec) {
    await delay(1);
  }
}

async function delay(ms) {
  return new Promise(resolve => setTimeout(resolve, ms));
}

function getDefaultButtonTexts(buttonCount) {
  const defaultButtonTexts = [];
  const totalButtons = buttonCount || 15;
  for (let i = 1; i <= totalButtons; i++) {
    defaultButtonTexts.push(`リンク${i}`);
  }
  return defaultButtonTexts;
}
