document.addEventListener('DOMContentLoaded', function () {
    const buttonTextInputs = document.getElementById('buttonTextInputs');
    const recoveryOptionRadioGroup = document.getElementById('recoveryOptionRadioGroup');

    // 回復オプションのラジオボタンを生成
    const addRecoveryOptionRadio = (label, value, checked) => {
        const radio = document.createElement('input');
        radio.type = 'radio';
        radio.name = 'recoveryOption';
        radio.value = value;
        radio.checked = checked;

        const labelElement = document.createElement('label');
        labelElement.textContent = label;

        // ラジオボタンとラベルをテーブルに追加
        const row = document.createElement('tr');
        const radioCell = document.createElement('td');
        const labelCell = document.createElement('td');
        radioCell.appendChild(radio);
        labelCell.appendChild(labelElement);
        row.appendChild(radioCell);
        row.appendChild(labelCell);
        recoveryOptionRadioGroup.appendChild(row);
    };

    // 回復ONと回復OFFのラジオボタンを生成
    chrome.storage.sync.get({ recoveryOption: 'on' }, obj => {
        addRecoveryOptionRadio('回復ON', 'on', obj.recoveryOption === 'on');
        addRecoveryOptionRadio('回復OFF', 'off', obj.recoveryOption === 'off');
    });


    // 保存ボタンのクリックイベント
    document.getElementById('saveButton').addEventListener('click', function () {
        // 回復オプション
        const recoveryOption = document.querySelector('input[name="recoveryOption"]:checked').value;
        // 回復後待機時間
        const waitTime = document.getElementById('waitTime').value;

        // ボタンテキストとURLを取得
        const buttonSettings = [];

        // ボタン数取得の共通処理
        const getButtonCount = (prefix) => {
            const buttonCountElement = document.getElementById(`buttonCount${prefix}`);
            return buttonCountElement ? buttonCountElement.value : 0;
        };

        const processButtons = (prefix) => {
            const buttonCount = getButtonCount(prefix);
            for (let i = 1; i <= buttonCount; i++) {
                const buttonTextElement = document.getElementById(`${prefix}${i}Text`);
                const buttonText = (buttonTextElement && buttonTextElement.value) || `リンク${prefix.charAt(prefix.length - 1)}${i}`;
                const buttonUrlElement = document.getElementById(`${prefix}${i}Url`);
                const buttonUrl = (buttonUrlElement && buttonUrlElement.value) || '';
                buttonSettings.push({ text: buttonText, url: buttonUrl });
            }
        };

        processButtons('B');
        processButtons('L');
        processButtons('R');


        // 待機時間、ボタン設定、回復オプションを保存
        chrome.storage.sync.set({
            buttonCountB, buttonCountL, buttonCountR,
            waitTime, buttonSettings, recoveryOption
        }, function () {
            console.log('Settings saved:', { buttonCountB, buttonCountL, buttonCountR, waitTime, buttonSettings, recoveryOption });
            displayStatusMessage('保存しました', true);
        });
    });


    // 保存されている待機時間とボタンのテキストを取得して表示
    chrome.storage.sync.get({
        buttonCountB: 5, buttonCountL: 10, buttonCountR: 10, waitTime: 300,
        buttonSettings: getDefaultButtonSettings(), recoveryOption: 'on'
    }, function (data) {
        document.getElementById('buttonCountB').value = data.buttonCountB;
        document.getElementById('buttonCountL').value = data.buttonCountL;
        document.getElementById('buttonCountR').value = data.buttonCountR;
        document.getElementById('waitTime').value = data.waitTime;


        // 回復オプションの値を設定
        const recoveryOptionRadio = document.querySelector(`input[name="recoveryOption"][value="${data.recoveryOption}"]`);
        if (recoveryOptionRadio) {
            recoveryOptionRadio.checked = true;
        } else {
            console.error('Recovery option radio button not found.');
        }



        setButtonSettings('B', data);
        setButtonSettings('L', data);
        setButtonSettings('R', data);

        // ボタン数を元に動的にボタン入力項目を生成
        generateButtonInputs(data.buttonCount);
    });

    // ボタンテキストとURLを設定
    const setButtonSettings = (prefix, data) => {
        const buttonCount = data[`buttonCount${prefix}`];
        for (let i = 1; i <= buttonCount; i++) {
            const buttonTextElement = document.getElementById(`button${prefix}${i}Text`);
            const buttonUrlElement = document.getElementById(`button${prefix}${i}Url`);

            if (buttonTextElement && buttonUrlElement) {
                buttonTextElement.value = data[`button${prefix}Settings`][i - 1].text;
                buttonUrlElement.value = data[`button${prefix}Settings`][i - 1].url;
            } else {
                //console.error(`Button text or URL element not found for button ${i}.`);
            }
        }
    };

    // 保存時のステータスメッセージ表示
    function displayStatusMessage(message, success) {
        const statusMessageElement = document.getElementById('statusMessage');
        statusMessageElement.textContent = message;
        statusMessageElement.style.color = success ? 'green' : 'red';
        setTimeout(() => {
            statusMessageElement.textContent = '';
        }, 3000);
    }


    // デフォルトのボタン設定を生成
    function getDefaultButtonSettings() {
        const buttonSettings = [];
        for (let i = 1; i <= 10; i++) {
            buttonSettings.push({ text: `リンク${i}`, url: '' });
        }
        return buttonSettings;
    }

    // ボタン数を元に動的にボタン入力項目を生成する関数
    function generateButtonInputs(prefix, buttonCount) {
        // 現在の入力項目をクリア
        const buttonTextInputs = document.getElementById(`buttonTextInputs${prefix}`);
        //buttonTextInputs.innerHTML = '';

        // 新しいボタンテキストとURLの入力フィールドを動的に生成
        for (let i = 1; i <= buttonCount; i++) {
            const buttonLabel = document.createElement('label');
            buttonLabel.textContent = `ボタン${i}`;
            buttonLabel.classList.add('option-button-label');

            const labelText = document.createElement('input');
            labelText.type = 'text';
            labelText.id = `button${prefix}${i}Text`;
            labelText.placeholder = `${buttonLabel.textContent}のテキスト`;

            const urlInput = document.createElement('textarea');
            urlInput.id = `button${prefix}${i}Url`;
            urlInput.placeholder = `${buttonLabel.textContent}のURL`;

            const row = document.createElement('tr');
            const labelAndTextCell = document.createElement('td');
            const urlCell = document.createElement('td');

            labelAndTextCell.appendChild(buttonLabel);
            labelAndTextCell.appendChild(labelText);
            row.appendChild(labelAndTextCell);
            buttonTextInputs.appendChild(row);

            const rowUrl = document.createElement('tr');
            const urlCellInner = document.createElement('td');
            urlCellInner.appendChild(urlInput);
            rowUrl.appendChild(urlCellInner);
            buttonTextInputs.appendChild(rowUrl);

            // 保存されているデータがあれば設定
            chrome.storage.sync.get({ [`button${prefix}Settings`]: getDefaultButtonSettings() }, function (data) {
                if (data[`button${prefix}Settings`][i - 1]) {
                    labelText.value = data[`button${prefix}Settings`][i - 1].text;
                    urlInput.value = data[`button${prefix}Settings`][i - 1].url;
                }
            });
        }
    }

    // 保存されている待機時間とボタンのテキストを取得して表示
    chrome.storage.sync.get({
        buttonCountB: 5, buttonCountL: 10, buttonCountR: 10, waitTime: 300,
        buttonSettings: getDefaultButtonSettings(), recoveryOption: 'on'
    }, function (data) {
        document.getElementById('buttonCountB').value = data.buttonCountB;
        document.getElementById('buttonCountL').value = data.buttonCountL;
        document.getElementById('buttonCountR').value = data.buttonCountR;
        document.getElementById('waitTime').value = data.waitTime;

        // 回復オプションの値を設定
        const recoveryOptionRadio = document.querySelector(`input[name="recoveryOption"][value="${data.recoveryOption}"]`);
        if (recoveryOptionRadio) {
            recoveryOptionRadio.checked = true;
        } else {
            console.error('Recovery option radio button not found.');
        }

        // ボタンテキストとURLを設定
        ['B', 'L', 'R'].forEach(prefix => {
            setButtonSettings(prefix, data);
            generateButtonInputs(prefix, data[`buttonCount${prefix}`]);
        });
    });


    // ボタン数を元に動的にボタン入力項目を生成する関数 old
    function generateButtonInputs_old(buttonCount) {
        // 現在の入力項目をクリア
        buttonTextInputs.innerHTML = '';

        // 新しいボタンテキストとURLの入力フィールドを動的に生成
        for (let i = 1; i <= buttonCount; i++) {
            const buttonLabel = document.createElement('label');
            buttonLabel.textContent = `ボタン${i}`;
            buttonLabel.classList.add('option-button-label');

            const labelText = document.createElement('input');
            labelText.type = 'text';
            labelText.id = `button${i}Text`;
            labelText.placeholder = `${buttonLabel.textContent}のテキスト`;

            const urlInput = document.createElement('textarea');
            urlInput.id = `button${i}Url`;
            urlInput.placeholder = `${buttonLabel.textContent}のURL`;

            const row = document.createElement('tr');
            const labelAndTextCell = document.createElement('td');
            const urlCell = document.createElement('td');

            labelAndTextCell.appendChild(buttonLabel);
            labelAndTextCell.appendChild(labelText);
            row.appendChild(labelAndTextCell);
            buttonTextInputs.appendChild(row);

            const rowUrl = document.createElement('tr');
            const urlCellInner = document.createElement('td');
            urlCellInner.appendChild(urlInput);
            rowUrl.appendChild(urlCellInner);
            buttonTextInputs.appendChild(rowUrl);

            // 保存されているデータがあれば設定
            chrome.storage.sync.get({ buttonSettings: getDefaultButtonSettings() }, function (data) {
                if (data.buttonSettings[i - 1]) {
                    labelText.value = data.buttonSettings[i - 1].text;
                    urlInput.value = data.buttonSettings[i - 1].url;
                }
            });
        }
    }

});
