document.addEventListener("DOMContentLoaded", async function () {
    const settingsContainer = document.getElementById("settingsContainer");
    const toggleAllButton = document.getElementById("toggleAllButton");

    // 複数の設定を生成する
    for (let settingId = 1; settingId <= 20; settingId++) {
        // 設定名を取得
        const settingName = await getSettingName(settingId);
        //console.log("settingName:",settingName);

        // チェックボックスの状態を取得
        const isEnabled = await getSettingEnabledStatus(settingId);

        // リンクを生成
        const settingLink = document.createElement("a");
        settingLink.href = `detail_options.html?settingId=${settingId}`;
        settingLink.textContent = settingName;

        // チェックボックスを生成
        const checkbox = document.createElement("input");
        checkbox.type = "checkbox";
        checkbox.checked = isEnabled;
        checkbox.dataset.settingId = settingId; // settingIdをデータ属性に追加

        // チェックボックスの状態が変更されたときの処理
        checkbox.addEventListener("change", async function () {
            // チェックボックスの状態を保存
            await setSettingEnabledStatus(settingId, checkbox.checked);
        });

        // 行を生成
        const row = document.createElement("tr");

        // チェックボックスをセルに追加
        const checkboxCell = document.createElement("td");
        checkboxCell.appendChild(checkbox);
        row.appendChild(checkboxCell);

        // リンクをセルに追加
        const linkCell = document.createElement("td");
        linkCell.appendChild(settingLink);
        row.appendChild(linkCell);

        // 行をコンテナに追加
        settingsContainer.appendChild(row);
    }

    // 一括でON/OFFするボタンのクリックイベントを追加
    toggleAllButton.addEventListener("click", async function () {
        const checkboxes = document.querySelectorAll('input[type="checkbox"]');
        const toggleValue = !checkboxes[0].checked; // 最初のチェックボックスの状態の反転

        // すべてのチェックボックスの状態を一括で設定
        checkboxes.forEach((checkbox) => {
            checkbox.checked = toggleValue;
            const settingId = parseInt(checkbox.dataset.settingId, 10);

            // チェックボックスの状態を保存
            setSettingEnabledStatus(settingId, toggleValue);
        });
    });

    (async () => {
        const openedTabs = []; // 開いたタブを保持するための配列
        const settingLink = `https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fmyp.aspx%3Frnd%3D497991&guid=ON`;

        async function checkIfTabExists(settingId) {
            return new Promise((resolve) => {
                chrome.tabs.query({}, async (tabs) => {
                    console.log("tabs:", tabs);
                    let tabExists = false;
                    for (const tab of tabs) {
                        const tabId = tab.id;
                        const storedSettingId = await getSettingId(tabId);
                        //console.log("Stored settingId:", storedSettingId);
                        if (storedSettingId === settingId) {
                            tabExists = true;
                        }
                    }

                    resolve(tabExists);
                });
            });
        }


        generateTabButton.addEventListener("click", async function () {
            for (let settingId = 1; settingId <= 20; settingId++) {
                const isEnabled = await getSettingEnabledStatus(settingId);

                if (isEnabled) {
                    const tabExists = await checkIfTabExists(settingId);

                    if (!tabExists) {
                        const tabs = await new Promise((resolve) => {
                            chrome.tabs.query({}, (tabs) => {
                                resolve(tabs);
                            });
                        });
                        chrome.tabs.create({ url: settingLink }, async (tab) => {
                            await chrome.storage.local.set({ [`settingId_${tab.id}`]: settingId });
                            console.log(`options local.set settingId=${settingId}`);
                            openedTabs.push(tab);
                            console.log("options local.set tab:", tab);

                        });
                    }
                }
            }
        });

        // オプションページを閉じるボタンの要素を取得
        const closeOptionsPageButton = document.getElementById("closeOptionsPage");

        // オプションページを閉じるボタンのクリックイベントを追加
        closeOptionsPageButton.addEventListener("click", function () {
            window.close(); // オプションページを閉じる
        });


        const closeAllTabsButton = document.getElementById("closeAllTabsButton");
        // 一括でタブを閉じるボタンのクリックイベントを追加
        closeAllTabsButton.addEventListener("click", async function () {
            for (const tab of openedTabs) {
                chrome.tabs.remove(tab.id);
            }
        });


        // バックアップボタンのクリックイベントを追加
        const backupButton = document.getElementById("backupButton");
        backupButton.addEventListener("click", async function () {
            const data = await getLocalData(); // データを非同期で取得
            // 特定のキーのデータだけを取得
            const keysToExport = ['Profile', ...Array.from({ length: 20 }, (_, i) => `sid${String(i + 1).padStart(2, '0')}`)];
            const dataToExport = Object.fromEntries(
                Object.entries(data).filter(([key]) => keysToExport.includes(key))
            );
            //console.log('エクスポートするデータ:', dataToExport);

            // 現在の日付を設定（YYYYMMDD形式）
            const currentDate = new Date().toISOString().slice(0, 10).replace(/-/g, '');
            // Manifestからextensionのversionを取得
            const extensionVersion = chrome.runtime.getManifest().version;
            // ダウンロード
            download(JSON.stringify({ version: extensionVersion, ...dataToExport }), `rise_backup_${currentDate}.conf`, 'application/json');
        });


        // バックアップデータをアップロードするボタンのクリックイベントを追加
        const uploadButton = document.getElementById('restoreButton');
        uploadButton.addEventListener('click', uploadBackupData);

    })();
});


// ストレージに保存しているデータ読み出し
async function getLocalData() {
    return new Promise((resolve) => {
        chrome.storage.local.get(null, function (result) {
            resolve(result);
        });
    });
}


// ダウンロード用の関数
function download(content, filename, contentType) {
    const a = document.createElement('a');
    const blob = new Blob([content], { type: contentType });
    a.href = URL.createObjectURL(blob);
    a.download = filename;
    a.click();
}


// バックアップデータをストレージにアップロードする
async function uploadBackupData() {
    const fileInput = document.createElement('input');
    fileInput.type = 'file';
    fileInput.accept = '.conf'; // アップロード可能な拡張子を指定

    // ファイルが選択されたときの処理
    fileInput.addEventListener('change', async (event) => {
        const file = event.target.files[0];
        if (file) {
            const fileReader = new FileReader();

            // ファイルの読み込みが完了したときの処理
            fileReader.onload = async (e) => {
                try {
                    const content = e.target.result;
                    const data = JSON.parse(content);
                    console.log("data:", data);

                    // ストレージを初期化
                    await chrome.storage.local.clear();
                    // 取得したデータをストレージに保存
                    await chrome.storage.local.set(data, () => {
                        console.log('アップロードが完了しました。');
                        //alert('アップロードが完了しました。');
                    });
                    // 画面をリロード
                    location.reload();

                } catch (error) {
                    console.error('データの解析に失敗しました。', error);
                    alert('データの解析に失敗しました。');
                }
            };
            // ファイルを読み込む
            fileReader.readAsText(file);
        }
    });

    // ファイル選択ダイアログを開く
    fileInput.click();
}


// 編成アップロード
document.getElementById("uploadFormationButton").addEventListener("click", async function () {
    const settingId = 110;
    const url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fuser%2Fequip_list_sp.aspx%3Fecat%3D0%26did%3D1%26crp%3D1%26rnd%3D432843&guid=ON";

    try {
        // タブ生成
        const tab = await new Promise((resolve) => {
            chrome.tabs.create({ url: url }, (tab) => {
                resolve(tab);
            });
        });
        await chrome.storage.local.set({ [`settingId_${tab.id}`]: settingId });

        await new Promise((resolve) => {
            chrome.tabs.onUpdated.addListener(function listener(tabId, changeInfo) {
                if (tabId === tab.id && changeInfo.status === 'complete') {
                    chrome.tabs.onUpdated.removeListener(listener);
                    resolve();
                }
            });
        });
        await bsleep(3000); // データの取得が完了するまで待つ

        // 所持アビリスト
        const cardDataKey = 'cardData';
        const cardData = await new Promise((resolve) => {
            chrome.storage.local.get([cardDataKey], function (result) {
                if (chrome.runtime.lastError) {
                    console.error(chrome.runtime.lastError);
                    resolve(null);
                } else {
                    resolve(result[cardDataKey]);
                }
            });
        });

        if (cardData) {
            console.log("saveFormationToCsv:", cardData);
            saveFormationToCsv(cardData);
            //displayMessage(`ボタンのデータを自動設定しました`, true);
        } else {
            console.error('formData from local storage:', cardData);
            displayMessage("データの読み込みに失敗しました", false);
        }

    } catch (error) {
        console.error("例外発生:", error);
        displayMessage("エラーが発生しました", error);
    }
});


// 編成カードデータをURLにCSV形式で保存する関数
async function saveFormationToCsv(cardData) {
    const profile = await getProfile();
    const uid = profile[0].uid;
    const gid = profile[0].gid;
    const uname = profile[0].uname;
    const fname = cardData[0].fname;
    const url = `${BASE_URL}/rise/form/upload/${gid}/${uid}/${uname}/${fname}`;
    console.log("url:",url);

    var csvText = "no,type,name,eid,rare,lv,bs,abi1,abi2,abi3,abi4,abi5,abi6,abi7,abi8\n";
    // カードデータをCSV形式に変換
    cardData.forEach(card => {
        // 各カードのデータをカンマで区切った形式に変換
        const abilities = card.abilities || [];
        const cardLine = `${card.no},${card.ctype},${card.name},${card.eid},${card.rare},${card.Lv},${card.bs},${abilities}\n`;
        csvText += cardLine;

        if (card.supportCards && card.supportCards.length > 0) {
            //console.log("length:", card.supportCards.length);
            //console.log("cards:", card.supportCards);
            card.supportCards.forEach(supportCard => {
                const abilitiesSupport = supportCard.abilities || [];
                csvText += `${card.no},${supportCard.type},${supportCard.name},${supportCard.eid},${supportCard.rare},"","",${abilitiesSupport}\n`;
            });
        }
    });
    //console.log("csv:", csvText);

    // BOM つきの UTF-8 ファイル生成
    const blob = createBOM_UTF8(csvText);
    const formData = new FormData();
    formData.append('file', blob, `${fname}.csv`);

    const xhr = new XMLHttpRequest();
    xhr.open('POST', url);
    xhr.send(formData);

    xhr.onload = function () {
        if (xhr.status === 200) {
            console.log('Settings saved in CSV format to server');
        } else {
            console.error('Failed to save settings:', xhr.statusText);
        }
    };

    xhr.onerror = function () {
        console.error('There was a problem with the XMLHttpRequest');
    };
}


async function getSettingId(tabId) {
    return new Promise((resolve) => {
        chrome.storage.local.get([`settingId_${tabId}`], (data) => {
            resolve(data[`settingId_${tabId}`]);
        });
    });
}


async function getSettingName(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.get({ [storageKey]: {} }, (data) => {
            const setting = data[storageKey];
            if (setting && setting.settingName) {
                resolve(setting.settingName);
            } else {
                resolve(`設定${settingId}`);
            }
        });
    });
}


async function getSettingEnabledStatus(settingId) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.get({ [storageKey]: {} }, (data) => {
            //console.log("getSettingEnabledStatus data:",data);
            const setting = data[storageKey];
            if (setting && setting.isEnabled !== undefined) {
                resolve(setting.isEnabled);
            } else {
                // デフォルトでは全ボタンが無効
                resolve(settingId === 1); // １つ目のボタンだけ有効にする場合
            }
        });
    });
}


async function setSettingEnabledStatus(settingId, isEnabled) {
    const storageKey = `sid${String(settingId).padStart(2, '0')}`;
    return new Promise((resolve) => {
        chrome.storage.local.get({ [storageKey]: {} }, (data) => {
            const setting = data[storageKey] || {};
            setting.isEnabled = isEnabled;
            chrome.storage.local.set({ [storageKey]: setting }, () => {
                resolve();
            });
        });
    });
}


// ライセンスチェック(uid、gid取得)
document.getElementById("checkLicenseButton").addEventListener("click", async function () {
    console.log("checkLicenseButton");
    const settingId = 200;
    const url = "https://g12010355.sp.pf.mbga.jp/?url=https%3A%2F%2Fmbga-sp.daisenran.jp%2Fuser%2Fprofile.aspx%3Frnd%3D0&guid=ON";

    try {
        const tab = await new Promise((resolve) => {
            chrome.tabs.create({ url: url }, (tab) => {
                resolve(tab);
            });
        });
        await chrome.storage.local.set({ [`settingId_${tab.id}`]: settingId });

        await new Promise((resolve) => {
            chrome.tabs.onUpdated.addListener(function listener(tabId, changeInfo) {
                if (tabId === tab.id && changeInfo.status === 'complete') {
                    chrome.tabs.onUpdated.removeListener(listener);
                    resolve();
                }
            });
        });
        await bsleep(400);

        const profile = await new Promise((resolve) => {
            chrome.storage.local.get('Profile', function (result) {
                const storedProfile = result['Profile'];
                console.log('Data retrieved from local storage:', storedProfile);
                resolve(storedProfile);
            });
        });

        console.log('Data retrieved from local storage:', profile);

        if (profile) {
            console.log('Data retrieved from local storage:', profile[0].uid);
            const ret = checkLicense(profile[0]);
            displayMessage(`認証が成功しました`, true);
        } else {
            displayMessage("認証に失敗しました", false);
        }
    } catch (error) {
        displayMessage("エラーが発生しました", error);
    }
});


function checkLicense(profile) {
    const url = `${BASE_URL}/rise/license/${profile.gid}/${profile.uid}/${profile.uname}`;
    console.log("url:", url);
    return fetch(url)
        .then((response) => {
            if (!response.ok) {
                throw new Error("Network response was not ok");
            }
            return response.text();
        })
        .then((csvText) => {

        })
        .catch((error) => {
            console.error("There was a problem with the fetch operation:", error);
        });
}


// リンクボタンがクリックされたときの処理
document.getElementById('browseFormationButton').addEventListener('click', async function () {
    try {
        const profile = await getProfile();
        const uid = profile[0].uid;
        const gid = profile[0].gid;
        const uname = profile[0].uname;
        const url = `${BASE_URL}/rise/form/${uid}`;
        chrome.tabs.create({ url: url });
    } catch (error) {
        console.error("Error fetching profile:", error);
        // エラーが発生した場合の処理を追加できます
    }
});
  